@@ -1,33 +1,14 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* libcdr
- * Version: MPL 1.1 / GPLv2+ / LGPLv2+
+/*
+ * This file is part of the libcdr project.
  *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License or as specified alternatively below. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * Major Contributor(s):
- * Copyright (C) 2012 Fridrich Strba <fridrich.strba@bluewin.ch>
- *
- *
- * All Rights Reserved.
- *
- * For minor contributions see the git repository.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPLv2+"), or
- * the GNU Lesser General Public License Version 2 or later (the "LGPLv2+"),
- * in which case the provisions of the GPLv2+ or the LGPLv2+ are applicable
- * instead of those above.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
 #include <math.h>
+#include <stack>
 #include "CDRStylesCollector.h"
 #include "CDRInternalStream.h"
 #include "libcdr_utils.h"
@@ -40,6 +21,7 @@
 #define DUMP_IMAGE 0
 #endif
 
+
 libcdr::CDRStylesCollector::CDRStylesCollector(libcdr::CDRParserState &ps) :
   m_ps(ps), m_page(8.5, 11.0, -4.25, -5.5), m_charStyles()
 {
@@ -49,23 +31,10 @@ libcdr::CDRStylesCollector::~CDRStylesCollector()
 {
 }
 
-void libcdr::CDRStylesCollector::collectFild(unsigned id, unsigned short fillType, const libcdr::CDRColor &color1, const libcdr::CDRColor &color2,
-    const libcdr::CDRGradient &gradient, const libcdr::CDRImageFill &imageFill)
-{
-  m_ps.m_fillStyles[id] = CDRFillStyle(fillType, color1, color2, gradient, imageFill);
-}
-
-void libcdr::CDRStylesCollector::collectOutl(unsigned id, unsigned short lineType, unsigned short capsType, unsigned short joinType, double lineWidth,
-    double stretch, double angle, const CDRColor &color, const std::vector<unsigned> &dashArray,
-    unsigned startMarkerId, unsigned endMarkerId)
-{
-  m_ps.m_lineStyles[id] = CDRLineStyle(lineType, capsType, joinType, lineWidth, stretch, angle, color, dashArray, startMarkerId, endMarkerId);
-}
-
 void libcdr::CDRStylesCollector::collectBmp(unsigned imageId, unsigned colorModel, unsigned width, unsigned height, unsigned bpp, const std::vector<unsigned> &palette, const std::vector<unsigned char> &bitmap)
 {
   libcdr::CDRInternalStream stream(bitmap);
-  WPXBinaryData image;
+  librevenge::RVNGBinaryData image;
 
   unsigned tmpPixelSize = (unsigned)(height * width);
   if (tmpPixelSize < (unsigned)height) // overflow
@@ -175,7 +144,7 @@ void libcdr::CDRStylesCollector::collectBmp(unsigned imageId, unsigned colorMode
   if (storeBMP)
   {
 #if DUMP_IMAGE
-    WPXString filename;
+    librevenge::RVNGString filename;
     filename.sprintf("bitmap%.8x.bmp", imageId);
     FILE *f = fopen(filename.cstr(), "wb");
     if (f)
@@ -193,9 +162,9 @@ void libcdr::CDRStylesCollector::collectBmp(unsigned imageId, unsigned colorMode
 
 void libcdr::CDRStylesCollector::collectBmp(unsigned imageId, const std::vector<unsigned char> &bitmap)
 {
-  WPXBinaryData image(&bitmap[0], bitmap.size());
+  librevenge::RVNGBinaryData image(&bitmap[0], bitmap.size());
 #if DUMP_IMAGE
-  WPXString filename;
+  librevenge::RVNGString filename;
   filename.sprintf("bitmap%.8x.bmp", imageId);
   FILE *f = fopen(filename.cstr(), "wb");
   if (f)
@@ -239,36 +208,29 @@ void libcdr::CDRStylesCollector::collectPaletteEntry(unsigned colorId, unsigned
   m_ps.m_documentPalette[colorId] = color;
 }
 
-void libcdr::CDRStylesCollector::collectFont(unsigned fontId, unsigned short, const WPXString &font)
-{
-  m_ps.m_fonts[fontId] = font;
-}
-
 void libcdr::CDRStylesCollector::collectText(unsigned textId, unsigned styleId, const std::vector<unsigned char> &data,
-    const std::vector<uint64_t> &charDescriptions, const std::map<unsigned, CDRCharacterStyle> &styleOverrides)
+                                             const std::vector<unsigned char> &charDescriptions, const std::map<unsigned, CDRCharacterStyle> &styleOverrides)
 {
   if (data.empty() || charDescriptions.empty())
     return;
 
-  WPXString text;
-  uint32_t tmpCharDescription = 0;
+  unsigned char tmpCharDescription = 0;
   unsigned i = 0;
   unsigned j = 0;
   std::vector<unsigned char> tmpTextData;
-  CDRCharacterStyle defaultCharStyle;
-  CDRCharacterStyle tmpCharStyle;
+  CDRCharacterStyle defaultCharStyle, tmpCharStyle;
+  getRecursedStyle(defaultCharStyle, styleId);
 
-  std::map<unsigned, CDRCharacterStyle>::const_iterator iter = m_charStyles.find(styleId);
-  if (iter != m_charStyles.end())
-    defaultCharStyle = iter->second;
+  CDRTextLine line;
   for (i=0, j=0; i<charDescriptions.size() && j<data.size(); ++i)
   {
     tmpCharStyle = defaultCharStyle;
-    iter = styleOverrides.find((tmpCharDescription >> 16) & 0xff);
+    std::map<unsigned, CDRCharacterStyle>::const_iterator iter = styleOverrides.find(tmpCharDescription & 0xfe);
     if (iter != styleOverrides.end())
       tmpCharStyle.overrideCharacterStyle(iter->second);
-    if ((uint32_t)(charDescriptions[i] & 0xffffff) != tmpCharDescription)
+    if (charDescriptions[i] != tmpCharDescription)
     {
+      librevenge::RVNGString text;
       if (!tmpTextData.empty())
       {
         if (tmpCharDescription & 0x01)
@@ -276,8 +238,10 @@ void libcdr::CDRStylesCollector::collectText(unsigned textId, unsigned styleId,
         else
           appendCharacters(text, tmpTextData, tmpCharStyle.m_charSet);
       }
+      line.append(CDRText(text, tmpCharStyle));
       tmpTextData.clear();
-      tmpCharDescription = (uint32_t)(charDescriptions[i] & 0xffffff);
+      tmpCharDescription = charDescriptions[i];
+
     }
     tmpTextData.push_back(data[j++]);
     if (tmpCharDescription & 0x01)
@@ -285,19 +249,49 @@ void libcdr::CDRStylesCollector::collectText(unsigned textId, unsigned styleId,
   }
   if (!tmpTextData.empty())
   {
+    librevenge::RVNGString text;
     if (tmpCharDescription & 0x01)
       appendCharacters(text, tmpTextData);
     else
       appendCharacters(text, tmpTextData, tmpCharStyle.m_charSet);
+    line.append(CDRText(text, tmpCharStyle));
+    CDR_DEBUG_MSG(("CDRStylesCollector::collectText - Text: %s\n", text.cstr()));
   }
 
-  CDR_DEBUG_MSG(("CDRStylesCollector::collectText - Text: %s\n", text.cstr()));
-  m_ps.m_texts[textId] = CDRText(text, tmpCharStyle);
+  std::vector<CDRTextLine> &paragraphVector = m_ps.m_texts[textId];
+  paragraphVector.push_back(line);
 }
 
-void libcdr::CDRStylesCollector::collectStlt(const std::map<unsigned, CDRCharacterStyle> &charStyles)
+void libcdr::CDRStylesCollector::collectStld(unsigned id, const CDRCharacterStyle &charStyle)
 {
-  m_charStyles = charStyles;
+  m_charStyles[id] = charStyle;
+}
+
+void libcdr::CDRStylesCollector::getRecursedStyle(CDRCharacterStyle &charStyle, unsigned styleId)
+{
+  std::map<unsigned, CDRCharacterStyle>::const_iterator iter = m_charStyles.find(styleId);
+  if (iter == m_charStyles.end())
+    return;
+
+  std::stack<CDRCharacterStyle> styleStack;
+  styleStack.push(iter->second);
+  if (iter->second.m_parentId)
+  {
+    std::map<unsigned, CDRCharacterStyle>::const_iterator iter2 = m_charStyles.find(iter->second.m_parentId);
+    while (iter2 != m_charStyles.end())
+    {
+      styleStack.push(iter2->second);
+      if (iter2->second.m_parentId)
+        iter2 = m_charStyles.find(iter2->second.m_parentId);
+      else
+        iter2 = m_charStyles.end();
+    }
+  }
+  while (!styleStack.empty())
+  {
+    charStyle.overrideCharacterStyle(styleStack.top());
+    styleStack.pop();
+  }
 }
 
 /* vim:set shiftwidth=2 softtabstop=2 expandtab: */
