@@ -1,33 +1,13 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* libcdr
- * Version: MPL 1.1 / GPLv2+ / LGPLv2+
+/*
+ * This file is part of the libcdr project.
  *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License or as specified alternatively below. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * Major Contributor(s):
- * Copyright (C) 2011 Fridrich Strba <fridrich.strba@bluewin.ch>
- *
- *
- * All Rights Reserved.
- *
- * For minor contributions see the git repository.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPLv2+"), or
- * the GNU Lesser General Public License Version 2 or later (the "LGPLv2+"),
- * in which case the provisions of the GPLv2+ or the LGPLv2+ are applicable
- * instead of those above.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include <libwpd-stream/libwpd-stream.h>
+#include <librevenge-stream/librevenge-stream.h>
 #include <locale.h>
 #include <math.h>
 #include <set>
@@ -59,7 +39,7 @@ libcdr::CMXParser::~CMXParser()
 {
 }
 
-bool libcdr::CMXParser::parseRecords(WPXInputStream *input, long size, unsigned level)
+bool libcdr::CMXParser::parseRecords(librevenge::RVNGInputStream *input, long size, unsigned level)
 {
   if (!input)
   {
@@ -69,7 +49,7 @@ bool libcdr::CMXParser::parseRecords(WPXInputStream *input, long size, unsigned
   long endPosition = -1;
   if (size > 0)
     endPosition = input->tell() + size;
-  while (!input->atEOS() && (endPosition < 0 || input->tell() < endPosition))
+  while (!input->isEnd() && (endPosition < 0 || input->tell() < endPosition))
   {
     if (!parseRecord(input, level))
       return false;
@@ -77,7 +57,7 @@ bool libcdr::CMXParser::parseRecords(WPXInputStream *input, long size, unsigned
   return true;
 }
 
-bool libcdr::CMXParser::parseRecord(WPXInputStream *input, unsigned level)
+bool libcdr::CMXParser::parseRecord(librevenge::RVNGInputStream *input, unsigned level)
 {
   if (!input)
   {
@@ -86,11 +66,11 @@ bool libcdr::CMXParser::parseRecord(WPXInputStream *input, unsigned level)
   try
   {
     m_collector->collectLevel(level);
-    while (!input->atEOS() && readU8(input) == 0)
+    while (!input->isEnd() && readU8(input) == 0)
     {
     }
-    if (!input->atEOS())
-      input->seek(-1, WPX_SEEK_CUR);
+    if (!input->isEnd())
+      input->seek(-1, librevenge::RVNG_SEEK_CUR);
     else
       return true;
     unsigned fourCC = readU32(input);
@@ -99,12 +79,12 @@ bool libcdr::CMXParser::parseRecord(WPXInputStream *input, unsigned level)
 
     CDR_DEBUG_MSG(("Record: level %u %s, length: 0x%.8x (%u)\n", level, toFourCC(fourCC), length, length));
 
-    if (fourCC == FOURCC_RIFF || fourCC == FOURCC_RIFX || fourCC == FOURCC_LIST)
+    if (fourCC == CDR_FOURCC_RIFF || fourCC == CDR_FOURCC_RIFX || fourCC == CDR_FOURCC_LIST)
     {
 #ifdef DEBUG
       unsigned listType = readU32(input);
 #else
-      input->seek(4, WPX_SEEK_CUR);
+      input->seek(4, librevenge::RVNG_SEEK_CUR);
 #endif
       CDR_DEBUG_MSG(("CMX listType: %s\n", toFourCC(listType)));
       unsigned dataSize = length-4;
@@ -115,7 +95,7 @@ bool libcdr::CMXParser::parseRecord(WPXInputStream *input, unsigned level)
       readRecord(fourCC, length, input);
 
     if (input->tell() < endPosition)
-      input->seek(endPosition, WPX_SEEK_SET);
+      input->seek(endPosition, librevenge::RVNG_SEEK_SET);
     return true;
   }
   catch (...)
@@ -124,33 +104,33 @@ bool libcdr::CMXParser::parseRecord(WPXInputStream *input, unsigned level)
   }
 }
 
-void libcdr::CMXParser::readRecord(unsigned fourCC, unsigned &length, WPXInputStream *input)
+void libcdr::CMXParser::readRecord(unsigned fourCC, unsigned &length, librevenge::RVNGInputStream *input)
 {
   long recordEnd = input->tell() + length;
   switch (fourCC)
   {
-  case FOURCC_cont:
+  case CDR_FOURCC_cont:
     readCMXHeader(input);
     break;
-  case FOURCC_DISP:
+  case CDR_FOURCC_DISP:
     readDisp(input, length);
     break;
-  case FOURCC_page:
+  case CDR_FOURCC_page:
     readPage(input, length);
     break;
-  case FOURCC_ccmm:
+  case CDR_FOURCC_ccmm:
     readCcmm(input, recordEnd);
     break;
   default:
     break;
   }
   if (input->tell() < recordEnd)
-    input->seek(recordEnd, WPX_SEEK_SET);
+    input->seek(recordEnd, librevenge::RVNG_SEEK_SET);
 }
 
-void libcdr::CMXParser::readCMXHeader(WPXInputStream *input)
+void libcdr::CMXParser::readCMXHeader(librevenge::RVNGInputStream *input)
 {
-  WPXString tmpString;
+  librevenge::RVNGString tmpString;
   unsigned i = 0;
   for (i = 0; i < 32; i++)
     tmpString.append((char)readU8(input));
@@ -194,45 +174,45 @@ void libcdr::CMXParser::readCMXHeader(WPXInputStream *input)
   CDR_DEBUG_MSG(("CMX Base Units: %u\n", m_unit));
   m_scale = readDouble(input, m_bigEndian);
   CDR_DEBUG_MSG(("CMX Units Scale: %.9f\n", m_scale));
-  input->seek(12, WPX_SEEK_CUR);
+  input->seek(12, librevenge::RVNG_SEEK_CUR);
   m_indexSectionOffset = readU32(input, m_bigEndian);
   m_infoSectionOffset = readU32(input, m_bigEndian);
   m_thumbnailOffset = readU32(input, m_bigEndian);
 #ifdef DEBUG
-  CDRBBox box = readBBox(input);
+  CDRBox box = readBBox(input);
 #endif
   CDR_DEBUG_MSG(("CMX Offsets: index section 0x%.8x, info section: 0x%.8x, thumbnail: 0x%.8x\n",
                  m_indexSectionOffset, m_infoSectionOffset, m_thumbnailOffset));
   CDR_DEBUG_MSG(("CMX Bounding Box: x: %f, y: %f, w: %f, h: %f\n", box.m_x, box.m_y, box.m_w, box.m_h));
 }
 
-void libcdr::CMXParser::readDisp(WPXInputStream *input, unsigned length)
+void libcdr::CMXParser::readDisp(librevenge::RVNGInputStream *input, unsigned length)
 {
-  WPXBinaryData previewImage;
-  previewImage.append(0x42);
-  previewImage.append(0x4d);
+  librevenge::RVNGBinaryData previewImage;
+  previewImage.append((unsigned char)0x42);
+  previewImage.append((unsigned char)0x4d);
 
   previewImage.append((unsigned char)((length+8) & 0x000000ff));
   previewImage.append((unsigned char)(((length+8) & 0x0000ff00) >> 8));
   previewImage.append((unsigned char)(((length+8) & 0x00ff0000) >> 16));
   previewImage.append((unsigned char)(((length+8) & 0xff000000) >> 24));
 
-  previewImage.append(0x00);
-  previewImage.append(0x00);
-  previewImage.append(0x00);
-  previewImage.append(0x00);
+  previewImage.append((unsigned char)0x00);
+  previewImage.append((unsigned char)0x00);
+  previewImage.append((unsigned char)0x00);
+  previewImage.append((unsigned char)0x00);
 
   long startPosition = input->tell();
-  input->seek(0x18, WPX_SEEK_CUR);
+  input->seek(0x18, librevenge::RVNG_SEEK_CUR);
   int lengthX = length + 10 - readU32(input);
-  input->seek(startPosition, WPX_SEEK_SET);
+  input->seek(startPosition, librevenge::RVNG_SEEK_SET);
 
   previewImage.append((unsigned char)((lengthX) & 0x000000ff));
   previewImage.append((unsigned char)(((lengthX) & 0x0000ff00) >> 8));
   previewImage.append((unsigned char)(((lengthX) & 0x00ff0000) >> 16));
   previewImage.append((unsigned char)(((lengthX) & 0xff000000) >> 24));
 
-  input->seek(4, WPX_SEEK_CUR);
+  input->seek(4, librevenge::RVNG_SEEK_CUR);
   for (unsigned i = 4; i<length; i++)
     previewImage.append(readU8(input));
 #if DUMP_PREVIEW_IMAGE
@@ -247,16 +227,16 @@ void libcdr::CMXParser::readDisp(WPXInputStream *input, unsigned length)
 #endif
 }
 
-void libcdr::CMXParser::readCcmm(WPXInputStream * /* input */, long &recordEnd)
+void libcdr::CMXParser::readCcmm(librevenge::RVNGInputStream * /* input */, long &recordEnd)
 {
   if (m_thumbnailOffset == (unsigned)-1)
     recordEnd += 0x10;
 }
 
-void libcdr::CMXParser::readPage(WPXInputStream *input, unsigned length)
+void libcdr::CMXParser::readPage(librevenge::RVNGInputStream *input, unsigned length)
 {
   long endPosition = length + input->tell();
-  while (!input->atEOS() && endPosition > input->tell())
+  while (!input->isEnd() && endPosition > input->tell())
   {
     long startPosition = input->tell();
     int instructionSize = readS16(input, m_bigEndian);
@@ -291,13 +271,13 @@ void libcdr::CMXParser::readPage(WPXInputStream *input, unsigned length)
     default:
       break;
     }
-    input->seek(m_nextInstructionOffset, WPX_SEEK_SET);
+    input->seek(m_nextInstructionOffset, librevenge::RVNG_SEEK_SET);
   }
 }
 
-void libcdr::CMXParser::readBeginPage(WPXInputStream *input)
+void libcdr::CMXParser::readBeginPage(librevenge::RVNGInputStream *input)
 {
-  CDRBBox box;
+  CDRBox box;
   CDRTransform matrix;
   unsigned flags = 0;
   if (m_precision == libcdr::PRECISION_32BIT)
@@ -318,7 +298,7 @@ void libcdr::CMXParser::readBeginPage(WPXInputStream *input)
       switch (tagId)
       {
       case CMX_Tag_BeginPage_PageSpecification:
-        input->seek(2, WPX_SEEK_CUR);
+        input->seek(2, librevenge::RVNG_SEEK_CUR);
         flags = readU32(input, m_bigEndian);
         box = readBBox(input);
         break;
@@ -328,13 +308,13 @@ void libcdr::CMXParser::readBeginPage(WPXInputStream *input)
       default:
         break;
       }
-      input->seek(startOffset + tagLength, WPX_SEEK_SET);
+      input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
     }
     while (tagId != CMX_Tag_EndTag);
   }
   else if (m_precision == libcdr::PRECISION_16BIT)
   {
-    input->seek(2, WPX_SEEK_CUR);
+    input->seek(2, librevenge::RVNG_SEEK_CUR);
     flags = readU32(input, m_bigEndian);
     box = readBBox(input);
   }
@@ -344,14 +324,14 @@ void libcdr::CMXParser::readBeginPage(WPXInputStream *input)
   m_collector->collectFlags(flags, true);
   m_collector->collectPageSize(box.getWidth(), box.getHeight(), box.getMinX(), box.getMinY());
 }
-void libcdr::CMXParser::readBeginLayer(WPXInputStream * /* input */)
+void libcdr::CMXParser::readBeginLayer(librevenge::RVNGInputStream * /* input */)
 {
 }
-void libcdr::CMXParser::readBeginGroup(WPXInputStream * /* input */)
+void libcdr::CMXParser::readBeginGroup(librevenge::RVNGInputStream * /* input */)
 {
 }
 
-void libcdr::CMXParser::readPolyCurve(WPXInputStream *input)
+void libcdr::CMXParser::readPolyCurve(librevenge::RVNGInputStream *input)
 {
   unsigned pointNum = 0;
   std::vector<std::pair<double, double> > points;
@@ -390,7 +370,7 @@ void libcdr::CMXParser::readPolyCurve(WPXInputStream *input)
       default:
         break;
       }
-      input->seek(startOffset + tagLength, WPX_SEEK_SET);
+      input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
     }
     while (tagId != CMX_Tag_EndTag);
   }
@@ -416,7 +396,7 @@ void libcdr::CMXParser::readPolyCurve(WPXInputStream *input)
   m_collector->collectLevel(1);
 }
 
-void libcdr::CMXParser::readEllipse(WPXInputStream *input)
+void libcdr::CMXParser::readEllipse(librevenge::RVNGInputStream *input)
 {
   double angle1 = 0.0;
   double angle2 = 0.0;
@@ -460,7 +440,7 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
       default:
         break;
       }
-      input->seek(startOffset + tagLength, WPX_SEEK_SET);
+      input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
     }
     while (tagId != CMX_Tag_EndTag);
   }
@@ -479,6 +459,7 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
     return;
 
   m_collector->collectObject(1);
+  CDRPath path;
   if (angle1 != angle2)
   {
     if (angle2 < angle1)
@@ -491,13 +472,13 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
 
     bool largeArc = (angle2 - angle1 > M_PI || angle2 - angle1 < -M_PI);
 
-    m_collector->collectMoveTo(x0, y0);
-    m_collector->collectArcTo(rx, ry, largeArc, true, x1, y1);
+    path.appendMoveTo(x0, y0);
+    path.appendArcTo(rx, ry, 0.0, largeArc, true, x1, y1);
     if (pie)
     {
-      m_collector->collectLineTo(cx, cy);
-      m_collector->collectLineTo(x0, y0);
-      m_collector->collectClosePath();
+      path.appendLineTo(cx, cy);
+      path.appendLineTo(x0, y0);
+      path.appendClosePath();
     }
   }
   else
@@ -508,15 +489,16 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
     double x1 = cx;
     double y1 = cy - ry;
 
-    m_collector->collectMoveTo(x0, y0);
-    m_collector->collectArcTo(rx, ry, false, true, x1, y1);
-    m_collector->collectArcTo(rx, ry, true, true, x0, y0);
+    path.appendMoveTo(x0, y0);
+    path.appendArcTo(rx, ry, 0.0, false, true, x1, y1);
+    path.appendArcTo(rx, ry, 0.0, true, true, x0, y0);
   }
+  m_collector->collectPath(path);
   m_collector->collectRotate(rotation, cx, cy);
   m_collector->collectLevel(1);
 }
 
-void libcdr::CMXParser::readRectangle(WPXInputStream *input)
+void libcdr::CMXParser::readRectangle(librevenge::RVNGInputStream *input)
 {
   double cx = 0.0;
   double cy = 0.0;
@@ -555,13 +537,13 @@ void libcdr::CMXParser::readRectangle(WPXInputStream *input)
       default:
         break;
       }
-      input->seek(startOffset + tagLength, WPX_SEEK_SET);
+      input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
     }
     while (tagId != CMX_Tag_EndTag);
   }
   else if (m_precision == libcdr::PRECISION_16BIT)
   {
-    input->seek(3, WPX_SEEK_CUR);
+    input->seek(3, librevenge::RVNG_SEEK_CUR);
     cx = readCoordinate(input, m_bigEndian);
     cy = readCoordinate(input, m_bigEndian);
     width = readCoordinate(input, m_bigEndian);
@@ -577,31 +559,33 @@ void libcdr::CMXParser::readRectangle(WPXInputStream *input)
   double y0 = cy - height / 2.0;
   double x1 = cx + width / 2.0;
   double y1 = cy + height / 2.0;
+  CDRPath path;
   if (radius > 0.0)
   {
-    m_collector->collectMoveTo(x0, y0-radius);
-    m_collector->collectLineTo(x0, y1+radius);
-    m_collector->collectQuadraticBezier(x0, y1, x0+radius, y1);
-    m_collector->collectLineTo(x1-radius, y1);
-    m_collector->collectQuadraticBezier(x1, y1, x1, y1+radius);
-    m_collector->collectLineTo(x1, y0-radius);
-    m_collector->collectQuadraticBezier(x1, y0, x1-radius, y0);
-    m_collector->collectLineTo(x0+radius, y0);
-    m_collector->collectQuadraticBezier(x0, y0, x0, y0-radius);
+    path.appendMoveTo(x0, y0-radius);
+    path.appendLineTo(x0, y1+radius);
+    path.appendQuadraticBezierTo(x0, y1, x0+radius, y1);
+    path.appendLineTo(x1-radius, y1);
+    path.appendQuadraticBezierTo(x1, y1, x1, y1+radius);
+    path.appendLineTo(x1, y0-radius);
+    path.appendQuadraticBezierTo(x1, y0, x1-radius, y0);
+    path.appendLineTo(x0+radius, y0);
+    path.appendQuadraticBezierTo(x0, y0, x0, y0-radius);
   }
   else
   {
-    m_collector->collectMoveTo(x0, y0);
-    m_collector->collectLineTo(x0, y1);
-    m_collector->collectLineTo(x1, y1);
-    m_collector->collectLineTo(x1, y0);
-    m_collector->collectLineTo(x0, y0);
+    path.appendMoveTo(x0, y0);
+    path.appendLineTo(x0, y1);
+    path.appendLineTo(x1, y1);
+    path.appendLineTo(x1, y0);
+    path.appendLineTo(x0, y0);
   }
+  m_collector->collectPath(path);
   m_collector->collectRotate(angle, cx, cy);
   m_collector->collectLevel(1);
 }
 
-libcdr::CDRTransform libcdr::CMXParser::readMatrix(WPXInputStream *input)
+libcdr::CDRTransform libcdr::CMXParser::readMatrix(librevenge::RVNGInputStream *input)
 {
   CDRTransform matrix;
   unsigned short type = readU16(input, m_bigEndian);
@@ -622,26 +606,26 @@ libcdr::CDRTransform libcdr::CMXParser::readMatrix(WPXInputStream *input)
   }
 }
 
-libcdr::CDRBBox libcdr::CMXParser::readBBox(WPXInputStream *input)
+libcdr::CDRBox libcdr::CMXParser::readBBox(librevenge::RVNGInputStream *input)
 {
   double x0 = readCoordinate(input, m_bigEndian);
   double y0 = readCoordinate(input, m_bigEndian);
   double x1 = readCoordinate(input, m_bigEndian);
   double y1 = readCoordinate(input, m_bigEndian);
-  CDRBBox box(x0, y0, x1, y1);
+  CDRBox box(x0, y0, x1, y1);
   return box;
 }
 
-void libcdr::CMXParser::readFill(WPXInputStream *input)
+void libcdr::CMXParser::readFill(librevenge::RVNGInputStream *input)
 {
-  unsigned char tagId = 0;
-  unsigned short tagLength = 0;
   unsigned fillIdentifier = readU16(input, m_bigEndian);
   switch (fillIdentifier)
   {
   case 1:
     if (m_precision == libcdr::PRECISION_32BIT)
     {
+      unsigned char tagId = 0;
+      unsigned short tagLength = 0;
       do
       {
         long startOffset = input->tell();
@@ -661,7 +645,7 @@ void libcdr::CMXParser::readFill(WPXInputStream *input)
         default:
           break;
         }
-        input->seek(startOffset + tagLength, WPX_SEEK_SET);
+        input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
       }
       while (tagId != CMX_Tag_EndTag);
     }
@@ -690,7 +674,7 @@ void libcdr::CMXParser::readFill(WPXInputStream *input)
   }
 }
 
-void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
+void libcdr::CMXParser::readRenderingAttributes(librevenge::RVNGInputStream *input)
 {
   unsigned char tagId = 0;
   unsigned short tagLength = 0;
@@ -716,7 +700,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
         default:
           break;
         }
-        input->seek(startOffset + tagLength, WPX_SEEK_SET);
+        input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
       }
       while (tagId != CMX_Tag_EndTag);
     }
@@ -741,17 +725,14 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
         switch (tagId)
         {
         case CMX_Tag_RenderAttr_OutlineSpec:
-          m_collector->collectOutlId(readU16(input, m_bigEndian));
           break;
         default:
           break;
         }
-        input->seek(startOffset + tagLength, WPX_SEEK_SET);
+        input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
       }
       while (tagId != CMX_Tag_EndTag);
     }
-    else if (m_precision == libcdr::PRECISION_16BIT)
-      m_collector->collectOutlId(readU16(input, m_bigEndian));
   }
   if (bitMask & 0x04) // lens
   {
@@ -773,7 +754,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
         default:
           break;
         }
-        input->seek(startOffset + tagLength, WPX_SEEK_SET);
+        input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
       }
       while (tagId != CMX_Tag_EndTag);
     }
@@ -798,7 +779,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
         default:
           break;
         }
-        input->seek(startOffset + tagLength, WPX_SEEK_SET);
+        input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
       }
       while (tagId != CMX_Tag_EndTag);
     }
@@ -823,14 +804,14 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
         default:
           break;
         }
-        input->seek(startOffset + tagLength, WPX_SEEK_SET);
+        input->seek(startOffset + tagLength, librevenge::RVNG_SEEK_SET);
       }
       while (tagId != CMX_Tag_EndTag);
     }
   }
 }
 
-void libcdr::CMXParser::readJumpAbsolute(WPXInputStream *input)
+void libcdr::CMXParser::readJumpAbsolute(librevenge::RVNGInputStream *input)
 {
   if (m_precision == libcdr::PRECISION_32BIT)
   {
@@ -854,7 +835,7 @@ void libcdr::CMXParser::readJumpAbsolute(WPXInputStream *input)
       default:
         break;
       }
-      input->seek(endOffset, WPX_SEEK_SET);
+      input->seek(endOffset, librevenge::RVNG_SEEK_SET);
     }
     while (tagId != CMX_Tag_EndTag);
   }
